//! Login to the NZZ archive and print the authentication cookie for further usage.
//!
//! Leveraging webdriver + geckodriver.

use std::time::Duration;

use anyhow::Result;
use cli::Config;
use fantoccini::{elements::Element, Client, ClientBuilder, Locator};
use serde_json::json;
use tokio::{sync::oneshot, time::sleep};

use crate::geckodriver::GECKODRIVER_HOST;

pub mod cli;
pub mod geckodriver;

const LOGIN_URL: &str = "https://zeitungsarchiv.nzz.ch/";

/// Entrypoint to login to the NZZ archive.
pub async fn run(args: Config, pw: &str) -> Result<()> {
    let (stop_tx, stop_rx) = oneshot::channel();
    let driver_handle = geckodriver::run(stop_rx).await?;

    let driver_args = json!({ "moz:firefoxOptions": {"args": ["-headless"]} });

    let client = ClientBuilder::native()
        .capabilities(driver_args.as_object().unwrap().clone())
        .connect(&format!("http://{GECKODRIVER_HOST}"))
        .await?;

    client.goto(LOGIN_URL).await?;

    sleep(Duration::from_millis(500)).await;

    // Sometimes cookie notice has to bee accepted, not sure when though
    if client.find(Locator::Css(".cmpboxbtncustom")).await.is_ok() {
        let fu_open_button: Element = element_from_css(&client, ".cmpboxbtncustom").await?;
        fu_open_button.click().await?;

        let fu_button: Element = element_from_css(&client, ".cmpboxbtn.cmpboxbtnyes.cmpboxbtnyescustomchoices.cmptxt_btn_save").await?;
        sleep(Duration::from_millis(500)).await;
        fu_button.click().await?;
    }


    let login_button: Element = element_from_css(&client, ".fup-menu-login-container").await?;
    sleep(Duration::from_millis(500)).await;
    login_button.click().await?;

    let login_iframe = element_from_css(&client, r#"iframe[id^="piano""#).await?;
    login_iframe.enter_frame().await?;

    let email_input = element_from_css(&client, r#"input[name="email"]"#).await?;
    email_input.send_keys(&args.username).await?;

    let pw_input: Element = element_from_css(&client, r#"input[type="password"]"#).await?;
    pw_input.send_keys(pw).await?;

    let submit = element_from_css(&client, r#"button[class="btn prime"]"#).await?;
    submit.click().await?;

    let main_frame = client.window().await?;
    client.switch_to_window(main_frame).await?;

    element_from_css(&client, ".fup-login-open.fup-button.fup-s-menu-login-open").await?;

    let cookies = client.get_all_cookies().await?;
    let cobbled_cookies = cookies
        .into_iter()
        .map(|cookie| format!("{}={}", cookie.name(), cookie.value()))
        .fold(String::new(), |mut acc, word| {
            if !acc.is_empty() {
                acc.push(';');
            }
            acc.push_str(&word);
            acc
        });
    println!("{cobbled_cookies}");

    client.close().await?;

    if let Some(driver_handle) = driver_handle {
        let _ = stop_tx.send(());
        driver_handle.abort();
    }

    Ok(())
}

async fn element_from_css(client: &Client, selector: &str) -> Result<Element> {
    Ok(client.wait().for_element(Locator::Css(selector)).await?)
}
